String clazzTpl({package}) => """
import 'dart:io';
import 'package:dio/dio.dart';
import 'package:dio/adapter.dart';
import 'package:$package/debug/app_debug_model.dart';
import 'package:$package/utils/toast_util.dart';
import 'package:$package/api/transformer/transformer.dart';
import 'package:$package/api/interceptors/log_interceptors.api.dart';
import 'package:$package/api/interceptors/request_interceptors.api.dart';
import 'package:$package/api/interceptors/response_interceptors.api.dart';
import 'package:$package/widget/base_store_container.dart';

/// 请叫我code哥
/// Generates Dart code
class HttpManager {
  static HttpManager _instance = HttpManager._internal();
  static Dio? _dio;
  static HttpClient? _client;

  factory HttpManager({required String host}) => _instance._host(host);

  HttpManager._internal() {
    if (null == _dio) {
      _dio = Dio(BaseOptions(
        contentType: Headers.jsonContentType,
        responseType: ResponseType.json,
        connectTimeout: 15000,
        receiveTimeout: 15000,
      ));
      _dio!.interceptors.add(RequestInterceptors());
      _dio!.interceptors.add(ResponseInterceptors());
      _dio!.interceptors.add(LogInterceptors());
      (_dio!.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
          (client) {
        // 设置代理
        String? proxyStr = _checkChangeProxy();
        if (proxyStr != null && proxyStr.isNotEmpty) {
          client.findProxy = (uri) {
            return proxyStr;
          };
        }
        //校验证书
        client.badCertificateCallback =
            (X509Certificate cert, String host, int port) {
          return true;
        };
        _client = client;
        return _client;
      };
    }
  }

  //用于指定特定域名
  HttpManager _host(String host) {
    if (_dio != null && _dio!.options.baseUrl != host) {
      _dio!.options.baseUrl = host;
    }
    return this;
  }

  Future<Response<dynamic>> request<T>(
    String path, {
    dynamic data,
    required Map<String, dynamic> queryParameters,
    required Options options,
    CancelToken? cancelToken,
    ProgressCallback? onSendProgress,
    ProgressCallback? onReceiveProgress,
  }) async {
    if (options.extra != null &&
        options.extra!.containsKey('isLoading') &&
        options.extra!['isLoading']) {
      Toast.showLoading();
    }
    Response resp;
    try {
      _checkChangeProxy();
      _checkChangeHost();
      resp = await _dio!.request(path,
          data: data,
          queryParameters: queryParameters,
          cancelToken: cancelToken,
          options: options,
          onSendProgress: onSendProgress,
          onReceiveProgress: onReceiveProgress);
    } on DioError catch (e) {
      resp = Response(requestOptions: e.requestOptions, data: e);
      Toast.showFail("网络出现错误");
    }
    if (!(resp.data is DioError) &&
        T.toString() != 'dynamic' &&
        T.toString() != 'void') {
      resp.data = await MyTransformer<T>().transformer(resp.data);
    }
    if (!(resp.data is DioError) &&
        T.toString() == 'dynamic' &&
        resp.data is String) {
      resp.data = await MyTransformer<T>().transformerString(resp.data);
    }
    if (resp.requestOptions.extra.containsKey('isLoading') &&
        resp.requestOptions.extra['isLoading']) {
      Toast.closeLoading();
    }
    // resp.data is DioError or BaseEntity<T> or dynamic
    return resp;
  }

  void _checkChangeHost() {
    // 处理人为切换环境
    final initConfig = StaticStore.store!.state.appConfigState!;
    AppDebugModel? item = initConfig.findAppDebugItem(AppDebugKey.hosts);
    if (item != null && item.hosts != null) {
      String selectHost =
          item.hosts!.firstWhere((element) => element.isDef).value;
      var isExists;
      try {
        // 是否包含,处理网络传其他host情况
        isExists = item.hosts!
            .firstWhere((element) => element.value == _dio!.options.baseUrl);
      } catch (e) {}
      if (_dio != null &&
          isExists != null &&
          selectHost != _dio!.options.baseUrl) {
        _dio!.options.baseUrl = selectHost;
      }
    }
  }

  String? _checkChangeProxy() {
    // 设置代理
    final initConfig = StaticStore.store!.state.appConfigState!;
    AppDebugModel? item = initConfig.findAppDebugItem(AppDebugKey.proxy);
    if (item != null &&
        item.desc != null &&
        item.desc!.isNotEmpty &&
        item.port != null &&
        item.port!.isNotEmpty) {
      if (_client != null) {
        _client!.findProxy = (uri) {
          return "PROXY \${item.desc}:\${item.port}";
        };
      }
      return "PROXY \${item.desc}:\${item.port}";
    } else {
      if (_client != null) {
        _client!.findProxy = null;
      }
    }
  }
}
""";
