String clazzTpl({package}) => """
import 'dart:io';
import 'dart:math';
import 'package:flutter/material.dart';
import 'package:$package/debug/app_debug_model.dart';
import 'package:$package/debug/slice/actions.dart';
import 'package:$package/utils/encrypt_util.dart';
import 'package:$package/widget/base_store_container.dart';
import 'package:device_info/device_info.dart';
import 'package:package_info/package_info.dart';

class AppAndDevice {
  static final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();

  ///指纹信息 设备唯一信息
  static String mur = '';
  static String deviceId = '';
  static String deviceType = '';
  static String deviceVersion = '';
  static String appVersion = '';
  static String appBuildNumber = '';

  static Future<void> loadDevice() async {
    if (deviceId.isEmpty ||
        deviceType.isEmpty ||
        deviceVersion.isEmpty ||
        appVersion.isEmpty ||
        appBuildNumber.isEmpty ||
        mur.isEmpty) {
      if (Platform.isAndroid) {
        AndroidDeviceInfo deviceData = await deviceInfoPlugin.androidInfo;
        deviceId = deviceData.androidId;
        deviceType = 'android';
        deviceVersion = deviceData.version.release;
      } else if (Platform.isIOS) {
        IosDeviceInfo deviceData = await deviceInfoPlugin.iosInfo;
        deviceId = deviceData.identifierForVendor;
        deviceType = 'iOS';
        deviceVersion = deviceData.systemVersion;
      }
      PackageInfo packageInfo = await PackageInfo.fromPlatform();
      appVersion = packageInfo.version;
      appBuildNumber = packageInfo.buildNumber;
      mur = EncryptUtil.encodeMD5("\${deviceId}_\${deviceType}_\$deviceVersion");
    }
  }
}

class AppConfigState {
  List<AppDebugModel>? appDebug;

  AppConfigState({this.appDebug});

  bool findAppDebug(AppDebugKey key) {
    bool on = false;
    appDebug?.forEach((element) {
      if (element.key == key) {
        on = element.on;
      }
    });
    return on;
  }

  String findAppHost() {
    try {
      final item =
          appDebug?.firstWhere((element) => element.key == AppDebugKey.hosts);
      return item!.hosts!.firstWhere((element) => element.isDef).value;
    } catch (e) {}
    return '';
  }

  AppDebugModel? findAppDebugItem(AppDebugKey key) {
    return appDebug?.firstWhere((element) => element.key == key);
  }

  static void themeNotify(BuildContext context) {
    if (StaticStore.store != null) {
      var item = StaticStore.store!.state.appConfigState!
          .findAppDebugItem(AppDebugKey.themes);
      if (item!.themes != null) {
        item.themes!.forEach((e1) {
          if (e1.isDef) {
            StaticStore.store!.dispatch(
                ThemeActions.updateTheme(ThemeState.formatThemeType(e1.value)));
          }
        });
      }
    }
  }

  void updateAppDebug(AppDebugModel model) {
    var state = StaticStore.store!.state.appConfigState!;
    appDebug = appDebug?.map((e) {
      if (e.key == model.key) {
        return model;
      }
      return e;
    }).toList();
    StaticStore.store!.dispatch(AppConfigActions.updateAppConfig(state));
  }

  static AppConfigState fromJson(dynamic json) {
    return AppConfigState(
      appDebug: (json["appDebug"] as List)
          .map((e) => AppDebugModel.fromJson(e))
          .toList(),
    );
  }

  dynamic toJson() => {
        'appDebug': appDebug?.map((v) => v.toJson()).toList(),
      };

  static AppConfigState initialState() {
    return AppConfigState(
      appDebug: [
        AppDebugModel(key: AppDebugKey.debugBanner, title: 'DebugBanner'),
        AppDebugModel(key: AppDebugKey.dev, title: '开发模式', on: false),
        AppDebugModel(key: AppDebugKey.policy, title: '隐私政策'),
        AppDebugModel(
          key: AppDebugKey.themes,
          title: '主题',
          themes: [
            DataModel(
              isDef: true,
              key: '浅色',
              value: 'light',
            ),
            DataModel(
              key: '深色',
              value: 'dark',
            ),
            DataModel(
              key: '跟随系统',
              value: 'system',
            ),
          ],
        ),
        AppDebugModel(
          key: AppDebugKey.websocket,
          title: 'websocket设置',
          desc: 'ws://10.100.0.133:9876/echo?id=\${Random().nextInt(10000)}',
        ),
        AppDebugModel(
          key: AppDebugKey.proxy,
          title: '代理设置',
        ),
        AppDebugModel(
          key: AppDebugKey.hosts,
          title: 'host设置',
          hosts: [
            DataModel(
              key: 'dev',
              value: 'https://cloudapi-wsdev.syitservice.com/',
            ),
            DataModel(
              key: 'test',
              value: 'https://cloudapi-wstest.syitservice.com/',
            ),
            DataModel(
              isDef: true,
              key: 'uat',
              value: 'https://cloudapi-wsuat.syitservice.com/',
            ),
            DataModel(
              key: 'prod',
              value: 'https://api.muxiaoguo.cn/api/',
            ),
          ],
        ),
        AppDebugModel(key: AppDebugKey.cleanMock, title: '清除mock数据'),
        AppDebugModel(key: AppDebugKey.errorLog, title: '错误日志'),
      ],
    );
  }
}
""";
